import * as i0 from '@angular/core';
import { Injectable, Component, Input, Directive, ViewChild, NgModule } from '@angular/core';
import * as i2 from '@angular/forms';
import { FormGroup, FormArray, FormControl, Validators, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { startWith, map } from 'rxjs';
import jsonata from 'jsonata';
import * as i1 from '@angular/common/http';
import { HttpClientModule } from '@angular/common/http';
import * as i1$1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i3 from '@angular/material/input';
import { MatInputModule } from '@angular/material/input';
import * as i4 from '@angular/material/form-field';
import * as i2$2 from '@angular/material/icon';
import { MatIconModule } from '@angular/material/icon';
import * as i3$1 from '@angular/material/button';
import { MatButtonModule } from '@angular/material/button';
import * as i2$1 from '@angular/material/checkbox';
import { MatCheckboxModule } from '@angular/material/checkbox';
import * as i4$1 from '@angular/material/select';
import { MatSelectModule } from '@angular/material/select';
import * as i5 from '@angular/material/core';
import { MatNativeDateModule } from '@angular/material/core';
import * as i4$2 from '@angular/material/datepicker';
import { MatDatepickerModule } from '@angular/material/datepicker';
import * as i6 from '@angular/material/autocomplete';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { ENTER, COMMA } from '@angular/cdk/keycodes';
import * as i4$3 from '@angular/material/chips';
import { MatChipsModule } from '@angular/material/chips';
import * as i1$2 from '@angular/material/tooltip';
import { MatTooltipModule } from '@angular/material/tooltip';
import * as i4$4 from '@angular/material/tabs';
import { MatTabsModule } from '@angular/material/tabs';

class JsonSchemaFormService {
    constructor() {
        /**
         * registry of custom widgets. The keys are the values used in schema.widgetType, the values
         * are the Type<any> of the custom widget component implementing WidgetComponent
         */
        this.registry = {};
    }
    /**
     * register custom component
     * @param key     the name of the component which is used in schema extension: widget=custom, widgetType=key
     * @param value   the implementation class
     */
    registerComponent(key, value) {
        this.registry[key] = value;
    }
}
JsonSchemaFormService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
JsonSchemaFormService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

/**
 * base component for all form elements
 */
class BaseComponent {
    constructor(http, service) {
        this.http = http;
        this.service = service;
        this.choices = [];
        /**
         * angular pipe sorting function for keyValue - keep the JSON order and do not
         * order alphabetically
         */
        this.originalOrder = (a, b) => {
            return 0;
        };
    }
    ngOnInit() {
        // make sure control matches the schema
        if (this.state.schema.type === 'object') {
            if (!(this.state.control instanceof FormGroup))
                throw new Error('For object schema, control must be a FormGroup');
        }
        else if (this.state.schema.type === 'array' && this.state.schema.layout !== 'select' && this.state.schema.layout !== 'chips') {
            if (!(this.state.control instanceof FormArray))
                throw new Error('For array schema, control must be a FormArray');
        }
        else {
            if (!(this.state.control instanceof FormControl))
                throw new Error('For primitive schema, control must be a FormControl');
        }
        // load choices
        if (this.state.schema.choicesUrl) {
            const request = this.state.schema.choicesVerb === 'GET' ?
                this.http.get(this.state.schema.choicesUrl) :
                this.http.post(this.state.schema.choicesUrl, {});
            request.subscribe((res) => {
                if (this.state.schema.jsonata) {
                    jsonata(this.state.schema.jsonata).evaluate(res).then(res2 => {
                        this.setChoices(res2);
                    });
                }
                else
                    this.setChoices(res);
            });
        }
        if (this.state.schema.enum) {
            // translate enum to choices
            this.setChoices(this.state.schema.enum);
        }
        else if (this.state.schema.choices) {
            this.setChoices(this.state.schema.choices);
        }
        else if (this.state.value)
            // default choice is current value
            this.setChoices([this.state.value]);
        // set name to title if set
        if (this.state.schema.title)
            this.state.name = this.state.schema.title;
    }
    setChoices(choices) {
        this.choices = [];
        for (const choice of choices)
            if (choice?.name && choice?.value)
                this.choices.push(choice);
            else
                this.choices.push({
                    value: choice,
                    name: choice ? choice : ''
                });
        this.filteredOptions = this.state.control.valueChanges.pipe(startWith(''), map(value => this._filter(value || '')));
    }
    _filter(value) {
        const filterValue = value.toLowerCase();
        return this.choices.filter(option => option.name.toLowerCase().includes(filterValue));
    }
    /**
     * cast control to primitive type
     */
    formControl() {
        return this.state.control;
    }
    /**
     * case control to group / object
     */
    formGroup() {
        return this.state.control;
    }
    /**
     * case control to array
     */
    formArray() {
        return this.state.control;
    }
    /**
     * create control and pass value to it
     */
    static createControl(schema, value, required) {
        if (schema.type === 'object') {
            return new FormGroup({});
        }
        if (schema.type === 'array' && schema.layout !== 'select' && schema.layout !== 'chips') {
            return new FormArray([]);
        }
        const control = new FormControl();
        this.prepareControl(control, schema, value, required);
        return control;
    }
    static prepareControl(control, schema, value, required) {
        // handle default
        if (schema.default)
            if (!value)
                value = schema.default;
        if (schema.type !== 'object' && schema.type !== 'array')
            control.setValue(value);
        if (required)
            control.addValidators(Validators.required);
        if (schema.pattern)
            control.addValidators(Validators.pattern(schema.pattern));
        if (schema.format)
            control.addValidators(Validators.pattern(this.formats[schema.format]));
        if (schema.maxLength)
            control.addValidators(Validators.maxLength(schema.maxLength));
        if (schema.minLength)
            control.addValidators(Validators.minLength(schema.minLength));
        if (schema.multipleOf)
            control.addValidators(multipleOf(schema.multipleOf));
        if (schema.maximum)
            control.addValidators(Validators.max(schema.maximum));
        if (schema.minimum)
            control.addValidators(Validators.min(schema.minimum));
        if (schema.exclusiveMaximum)
            control.addValidators(Validators.max(schema.exclusiveMaximum - 1));
        if (schema.exclusiveMinimum)
            control.addValidators(Validators.min(schema.exclusiveMinimum + 1));
        if (schema.maxItems)
            control.addValidators(maxItems(schema.maxItems));
        if (schema.minItems)
            control.addValidators(minItems(schema.minItems));
        if (schema.maxProperties)
            control.addValidators(maxProperties(schema.maxProperties));
        if (schema.minProperties)
            control.addValidators(minProperties(schema.minProperties));
        if (schema.uniqueItems)
            control.addValidators(uniqueItems());
        if (schema.propertyNames)
            control.addValidators(propertyNames(schema.propertyNames));
        if (schema.dependencies)
            control.addValidators(dependencies(schema.dependencies));
        if (schema.readOnly)
            control.disable();
        if (schema.createOnly && value)
            control.disable();
        return control;
    }
}
/**
 * built-in formats
 */
BaseComponent.formats = {
    email: /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/,
    ipv4: /^(?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d)(\.|$)){4}$/,
    url: /^https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_\+.~#?&//=]*)$/,
    uri: /^\w+:(\/?\/?)[^\s]+$/
};
BaseComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: BaseComponent, deps: [{ token: i1.HttpClient }, { token: JsonSchemaFormService }], target: i0.ɵɵFactoryTarget.Component });
BaseComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: BaseComponent, selector: "app-base", inputs: { state: "state" }, ngImport: i0, template: "<!--\n\n<app-input *ngIf=\"state.schema.type === 'string'\" [state]=\"state\"></app-input>\n<app-boolean *ngIf=\"state.schema.type === 'boolean'\" [state]=\"state\"></app-boolean>\n<app-object *ngIf=\"state.schema.type === 'object'\" [state]=\"state\"></app-object>\n<app-array *ngIf=\"state.schema.type === 'array'\" [state]=\"state\"></app-array>\n\n-->", styles: [""] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: BaseComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-base', template: "<!--\n\n<app-input *ngIf=\"state.schema.type === 'string'\" [state]=\"state\"></app-input>\n<app-boolean *ngIf=\"state.schema.type === 'boolean'\" [state]=\"state\"></app-boolean>\n<app-object *ngIf=\"state.schema.type === 'object'\" [state]=\"state\"></app-object>\n<app-array *ngIf=\"state.schema.type === 'array'\" [state]=\"state\"></app-array>\n\n-->" }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }, { type: JsonSchemaFormService }]; }, propDecorators: { state: [{
                type: Input
            }] } });
function multipleOf(multiple) {
    return (control) => {
        if (control.value % multiple === 0)
            return null;
        else
            return { multipleOf: { value: control.value } };
    };
}
function maxItems(max) {
    return (control) => {
        if (control.value.length <= max)
            return null;
        else
            return { maxItems: { value: control.value } };
    };
}
function minItems(min) {
    return (control) => {
        if (control.value.length >= min)
            return null;
        else
            return { minItems: { value: control.value } };
    };
}
function maxProperties(max) {
    return (control) => {
        if (Object.keys(control.value).length <= max)
            return null;
        else
            return { maxProperties: { value: control.value } };
    };
}
function minProperties(min) {
    return (control) => {
        if (Object.keys(control.value).length >= min)
            return null;
        else
            return { minProperties: { value: control.value } };
    };
}
function uniqueItems() {
    return (control) => {
        const unique = [];
        for (const i of control.value)
            if (!unique.includes(i))
                unique.push(i);
        if (unique.length === control.value.length)
            return null;
        else
            return { minItems: { value: control.value } };
    };
}
function propertyNames(pattern) {
    return (control) => {
        for (const key of Object.keys(control.value)) {
            const re = new RegExp(pattern);
            if (!re.test(key))
                return { propertyNames: { value: control.value } };
        }
        ;
        return null;
    };
}
function dependencies(map) {
    return (control) => {
        for (const [k, v] of Object.entries(map))
            if (control.value[k])
                for (const i of v)
                    if (!control.value[i])
                        return { propertyNames: { value: control.value } };
        return null;
    };
}

/**
 * directive to dynamically add form elements
 */
class CompDirective {
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
}
CompDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: CompDirective, deps: [{ token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
CompDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "15.2.3", type: CompDirective, selector: "[compHost]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: CompDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[compHost]',
                }]
        }], ctorParameters: function () { return [{ type: i0.ViewContainerRef }]; } });

/**
 * use checkbox for boolean
 */
class BooleanComponent extends BaseComponent {
}
BooleanComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: BooleanComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
BooleanComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: BooleanComponent, selector: "app-boolean", usesInheritance: true, ngImport: i0, template: "<mat-checkbox [formControl]=\"formControl()\">{{state.name}}</mat-checkbox>", styles: [""], dependencies: [{ kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "component", type: i2$1.MatCheckbox, selector: "mat-checkbox", inputs: ["disableRipple", "color", "tabIndex"], exportAs: ["matCheckbox"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: BooleanComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-boolean', template: "<mat-checkbox [formControl]=\"formControl()\">{{state.name}}</mat-checkbox>" }]
        }] });

/**
 * single text field inputs
 */
class InputComponent extends BaseComponent {
    /**
     * called from template in the "simple" type. If "type" is "number" or "integer",
     * the HTML input type is "number" which avoids normal string input
     */
    getInputType() {
        if (this.state.schema.type === 'number') {
            return 'number';
        }
        if (this.state.schema.type === 'integer') {
            return 'number';
        }
        return this.state.schema.widget ? this.state.schema.widget : 'string';
    }
}
InputComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: InputComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
InputComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: InputComponent, selector: "app-input", usesInheritance: true, ngImport: i0, template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <input *ngIf=\"getInputType() !== 'number'\" matInput [type]=\"getInputType()\" [formControl]=\"formControl()\"\n        [placeholder]=\"state.schema.examples && state.schema.examples[0] ? state.schema.examples[0] : ''\">\n    <input *ngIf=\"getInputType() === 'number'\" matInput type=\"number\" [formControl]=\"formControl()\"\n        [placeholder]=\"state.schema.examples && state.schema.examples[0] ? state.schema.examples[0] : ''\">\n    <mat-error *ngIf=\"state.schema.errorMessage\">{{state.schema.errorMessage}}</mat-error>\n</mat-form-field>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.NumberValueAccessor, selector: "input[type=number][formControlName],input[type=number][formControl],input[type=number][ngModel]" }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "directive", type: i3.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }, { kind: "directive", type: i4.MatError, selector: "mat-error, [matError]", inputs: ["id"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: InputComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-input', template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <input *ngIf=\"getInputType() !== 'number'\" matInput [type]=\"getInputType()\" [formControl]=\"formControl()\"\n        [placeholder]=\"state.schema.examples && state.schema.examples[0] ? state.schema.examples[0] : ''\">\n    <input *ngIf=\"getInputType() === 'number'\" matInput type=\"number\" [formControl]=\"formControl()\"\n        [placeholder]=\"state.schema.examples && state.schema.examples[0] ? state.schema.examples[0] : ''\">\n    <mat-error *ngIf=\"state.schema.errorMessage\">{{state.schema.errorMessage}}</mat-error>\n</mat-form-field>" }]
        }] });

/**
 * select input
 */
class SelectComponent extends BaseComponent {
}
SelectComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: SelectComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
SelectComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: SelectComponent, selector: "app-select", usesInheritance: true, ngImport: i0, template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <mat-select [formControl]=\"formControl()\" [multiple]=\"state.schema.type === 'array'\">\n        <mat-option *ngFor=\"let choice of choices\" [value]=\"choice.value\">{{choice.name}}</mat-option>\n    </mat-select>\n</mat-form-field>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }, { kind: "component", type: i4$1.MatSelect, selector: "mat-select", inputs: ["disabled", "disableRipple", "tabIndex", "hideSingleSelectionIndicator"], exportAs: ["matSelect"] }, { kind: "component", type: i5.MatOption, selector: "mat-option", exportAs: ["matOption"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: SelectComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-select', template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <mat-select [formControl]=\"formControl()\" [multiple]=\"state.schema.type === 'array'\">\n        <mat-option *ngFor=\"let choice of choices\" [value]=\"choice.value\">{{choice.name}}</mat-option>\n    </mat-select>\n</mat-form-field>" }]
        }] });

class UploadComponent extends BaseComponent {
    /**
     * allows for the result of a file upload to be written into a text form element
     */
    handleFileInput(base64, event) {
        if (10 * 1024 * 1024 <= event.target.files.item(0).size) {
            console.log('The file size is limited to 10MB');
            return;
        }
        let value;
        const reader = new FileReader();
        reader.onload = () => {
            value = reader.result;
            if (this.state.schema.type === 'array') {
                value = JSON.parse(value);
            }
            if (this.state.schema.type === 'object') {
                try {
                    value = JSON.parse(value);
                }
                catch (ignore) {
                }
                value = {
                    name: event.target.files.item(0).name,
                    lastModified: event.target.files.item(0).lastModified,
                    size: event.target.files.item(0).size,
                    type: event.target.files.item(0).type,
                    value: value
                };
            }
            this.state.control.setValue(value);
        };
        if (base64) {
            reader.readAsDataURL(event.target.files.item(0));
        }
        else {
            reader.readAsText(event.target.files.item(0));
        }
    }
}
UploadComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: UploadComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
UploadComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: UploadComponent, selector: "app-upload", usesInheritance: true, ngImport: i0, template: "<!-- upload -->\n<div *ngIf=\"state.schema.widget === 'upload'\">\n    <input type=\"file\" id=\"file\" (change)=\"handleFileInput(false, $event)\">\n</div>\n\n<!-- upload64 -->\n<div *ngIf=\"state.schema.widget === 'upload64'\">\n    <input type=\"file\" id=\"file\" (change)=\"handleFileInput(true, $event)\">\n</div>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: UploadComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-upload', template: "<!-- upload -->\n<div *ngIf=\"state.schema.widget === 'upload'\">\n    <input type=\"file\" id=\"file\" (change)=\"handleFileInput(false, $event)\">\n</div>\n\n<!-- upload64 -->\n<div *ngIf=\"state.schema.widget === 'upload64'\">\n    <input type=\"file\" id=\"file\" (change)=\"handleFileInput(true, $event)\">\n</div>" }]
        }] });

class DateComponent extends BaseComponent {
    ngOnInit() {
        if (this.state.schema.type === 'integer') {
            // create a new control and convert between timestamp and Date
            this.control = new FormControl(new Date(this.state.value));
            this.control.valueChanges.subscribe(res => {
                this.state.control.setValue(res instanceof Date ? res.valueOf() : res);
            });
        }
        else if (this.state.schema.dateFormat) {
            // create a new control and convert between Date format string and Date
            this.control = this.state.control;
            const pdate = this.state.value.split(this.getDelimiter(this.state.schema.dateFormat));
            const pformat = this.state.schema.dateFormat.split(this.getDelimiter(this.state.schema.dateFormat));
            this.control = new FormControl(new Date(pdate[pformat.indexOf('yyyy')], pdate[pformat.indexOf('MM')] - 1, pdate[pformat.indexOf('dd')]));
            this.control.valueChanges.subscribe(date => {
                if (!date) {
                    this.state.control.setValue(null);
                    return;
                }
                const pformat = this.state.schema.dateFormat.split(this.getDelimiter(this.state.schema.dateFormat));
                const pdate = [null, null, null];
                pdate[pformat.indexOf('yyyy')] = date.getFullYear();
                pdate[pformat.indexOf('MM')] = date.getMonth() + 1;
                pdate[pformat.indexOf('dd')] = date.getDate();
                this.state.control.setValue(pdate[0] + this.getDelimiter(this.state.schema.dateFormat) + pdate[1] + this.getDelimiter(this.state.schema.dateFormat) + pdate[2]);
            });
        }
        else
            // use state.control directly
            this.control = this.state.control;
    }
    /**
     * find the first non letter character in a date format such as dd/MM/yyyy (returns /)
     */
    getDelimiter(format) {
        const delim = format.match(/\W/g);
        if (!delim?.[0]) {
            throw new Error('No delimiter found in date format: ' + format);
        }
        return delim[0];
    }
}
DateComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: DateComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
DateComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: DateComponent, selector: "app-date", usesInheritance: true, ngImport: i0, template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <input matInput [matDatepicker]=\"picker\" [formControl]=\"control\">\n    <mat-datepicker-toggle matIconSuffix [for]=\"picker\"></mat-datepicker-toggle>\n    <mat-datepicker #picker></mat-datepicker>\n</mat-form-field>", styles: [""], dependencies: [{ kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "directive", type: i3.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }, { kind: "directive", type: i4.MatSuffix, selector: "[matSuffix], [matIconSuffix], [matTextSuffix]", inputs: ["matTextSuffix"] }, { kind: "component", type: i4$2.MatDatepicker, selector: "mat-datepicker", exportAs: ["matDatepicker"] }, { kind: "directive", type: i4$2.MatDatepickerInput, selector: "input[matDatepicker]", inputs: ["matDatepicker", "min", "max", "matDatepickerFilter"], exportAs: ["matDatepickerInput"] }, { kind: "component", type: i4$2.MatDatepickerToggle, selector: "mat-datepicker-toggle", inputs: ["for", "tabIndex", "aria-label", "disabled", "disableRipple"], exportAs: ["matDatepickerToggle"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: DateComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-date', template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <input matInput [matDatepicker]=\"picker\" [formControl]=\"control\">\n    <mat-datepicker-toggle matIconSuffix [for]=\"picker\"></mat-datepicker-toggle>\n    <mat-datepicker #picker></mat-datepicker>\n</mat-form-field>" }]
        }] });

class TextareaComponent extends BaseComponent {
}
TextareaComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: TextareaComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TextareaComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: TextareaComponent, selector: "app-textarea", usesInheritance: true, ngImport: i0, template: "<mat-form-field [ngStyle]=\"state.schema.style\" [ngClass]=\"state.schema.class\">\n    <mat-label>{{state.name}}</mat-label>\n    <textarea matInput [formControl]=\"formControl()\"></textarea>\n</mat-form-field>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1$1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "directive", type: i3.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: TextareaComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-textarea', template: "<mat-form-field [ngStyle]=\"state.schema.style\" [ngClass]=\"state.schema.class\">\n    <mat-label>{{state.name}}</mat-label>\n    <textarea matInput [formControl]=\"formControl()\"></textarea>\n</mat-form-field>" }]
        }] });

class AutocompleteComponent extends BaseComponent {
}
AutocompleteComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: AutocompleteComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AutocompleteComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: AutocompleteComponent, selector: "app-autocomplete", usesInheritance: true, ngImport: i0, template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <input type=\"text\" matInput [formControl]=\"formControl()\" [matAutocomplete]=\"auto\">\n    <mat-autocomplete autoActiveFirstOption #auto=\"matAutocomplete\">\n        <mat-option *ngFor=\"let option of filteredOptions | async\" [value]=\"option.value\">\n            {{option.name}}\n        </mat-option>\n    </mat-autocomplete>\n</mat-form-field>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "directive", type: i3.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }, { kind: "component", type: i5.MatOption, selector: "mat-option", exportAs: ["matOption"] }, { kind: "component", type: i6.MatAutocomplete, selector: "mat-autocomplete", inputs: ["disableRipple", "hideSingleSelectionIndicator"], exportAs: ["matAutocomplete"] }, { kind: "directive", type: i6.MatAutocompleteTrigger, selector: "input[matAutocomplete], textarea[matAutocomplete]", exportAs: ["matAutocompleteTrigger"] }, { kind: "pipe", type: i1$1.AsyncPipe, name: "async" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: AutocompleteComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-autocomplete', template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <input type=\"text\" matInput [formControl]=\"formControl()\" [matAutocomplete]=\"auto\">\n    <mat-autocomplete autoActiveFirstOption #auto=\"matAutocomplete\">\n        <mat-option *ngFor=\"let option of filteredOptions | async\" [value]=\"option.value\">\n            {{option.name}}\n        </mat-option>\n    </mat-autocomplete>\n</mat-form-field>" }]
        }] });

class ChipsComponent extends BaseComponent {
    constructor() {
        super(...arguments);
        this.addOnBlur = true;
        this.separatorKeysCodes = [ENTER, COMMA];
    }
    ngOnInit() {
        super.ngOnInit();
        if (!this.state.value)
            this.state.value = [];
    }
    add(event) {
        const value = (event.value || '').trim();
        // Add our fruit
        if (value) {
            this.state.value.push(value);
        }
        // Clear the input value
        event.chipInput.clear();
        this.state.control.setValue(this.state.value);
    }
    remove(fruit) {
        const index = this.state.value.indexOf(fruit);
        if (index >= 0) {
            this.state.value.splice(index, 1);
        }
        this.state.control.setValue(this.state.value);
    }
    edit(fruit, event) {
        const value = event.value.trim();
        // Remove fruit if it no longer has a name
        if (!value) {
            this.remove(fruit);
            return;
        }
        // Edit existing fruit
        const index = this.state.value.indexOf(fruit);
        if (index >= 0) {
            this.state.value[index] = value;
        }
        this.state.control.setValue(this.state.value);
    }
}
ChipsComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: ChipsComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
ChipsComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: ChipsComponent, selector: "app-chips", usesInheritance: true, ngImport: i0, template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <mat-chip-grid #chipGrid>\n        <mat-chip-row *ngFor=\"let fruit of state.value\" (removed)=\"remove(fruit)\" [editable]=\"true\"\n            (edited)=\"edit(fruit, $event)\" [aria-description]=\"'press enter to edit ' + fruit\">\n            {{fruit}}\n            <button matChipRemove [attr.aria-label]=\"'remove ' + fruit\">\n                <mat-icon>cancel</mat-icon>\n            </button>\n        </mat-chip-row>\n        <input [matChipInputFor]=\"chipGrid\" [matChipInputSeparatorKeyCodes]=\"separatorKeysCodes\"\n            [matChipInputAddOnBlur]=\"addOnBlur\" (matChipInputTokenEnd)=\"add($event)\" />\n    </mat-chip-grid>\n</mat-form-field>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }, { kind: "component", type: i2$2.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "component", type: i4$3.MatChipGrid, selector: "mat-chip-grid", inputs: ["tabIndex", "disabled", "placeholder", "required", "value", "errorStateMatcher"], outputs: ["change", "valueChange"] }, { kind: "directive", type: i4$3.MatChipInput, selector: "input[matChipInputFor]", inputs: ["matChipInputFor", "matChipInputAddOnBlur", "matChipInputSeparatorKeyCodes", "placeholder", "id", "disabled"], outputs: ["matChipInputTokenEnd"], exportAs: ["matChipInput", "matChipInputFor"] }, { kind: "directive", type: i4$3.MatChipRemove, selector: "[matChipRemove]" }, { kind: "component", type: i4$3.MatChipRow, selector: "mat-chip-row, [mat-chip-row], mat-basic-chip-row, [mat-basic-chip-row]", inputs: ["color", "disabled", "disableRipple", "tabIndex", "editable"], outputs: ["edited"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: ChipsComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-chips', template: "<mat-form-field>\n    <mat-label>{{state.name}}</mat-label>\n    <mat-chip-grid #chipGrid>\n        <mat-chip-row *ngFor=\"let fruit of state.value\" (removed)=\"remove(fruit)\" [editable]=\"true\"\n            (edited)=\"edit(fruit, $event)\" [aria-description]=\"'press enter to edit ' + fruit\">\n            {{fruit}}\n            <button matChipRemove [attr.aria-label]=\"'remove ' + fruit\">\n                <mat-icon>cancel</mat-icon>\n            </button>\n        </mat-chip-row>\n        <input [matChipInputFor]=\"chipGrid\" [matChipInputSeparatorKeyCodes]=\"separatorKeysCodes\"\n            [matChipInputAddOnBlur]=\"addOnBlur\" (matChipInputTokenEnd)=\"add($event)\" />\n    </mat-chip-grid>\n</mat-form-field>" }]
        }] });

/**
 * determine which form element to use and render it dynamically
 */
class WrapperComponent extends BaseComponent {
    /**
     * determine which form element to use
     */
    ngOnInit() {
        const viewContainerRef = this.compHost.viewContainerRef;
        viewContainerRef.clear();
        let type;
        if (this.state.schema.additionalProperties)
            type = WrapperComponent.additionalPropertiesComponent;
        else if (this.state.schema.layout === 'tab')
            type = WrapperComponent.tabComponent;
        else if (this.state.schema.layout === 'table')
            type = WrapperComponent.tableComponent;
        else if (this.state.schema.layout === 'select')
            type = SelectComponent;
        else if (this.state.schema.layout === 'chips')
            type = ChipsComponent;
        else if (this.state.schema.type === 'object')
            type = WrapperComponent.objectComponent;
        else if (this.state.schema.type === 'array')
            type = WrapperComponent.arrayComponent;
        else if (this.state.schema.widget === 'select')
            type = SelectComponent;
        else if (this.state.schema.choicesUrl || this.state.schema.choices)
            type = AutocompleteComponent;
        else if (this.state.schema.enum)
            type = SelectComponent;
        else if (this.state.schema.widget === 'custom')
            type = this.service.registry[this.state.schema.widgetType];
        else if (this.state.schema.widget === 'textarea')
            type = TextareaComponent;
        else if (this.state.schema.widget === 'date')
            type = DateComponent;
        else if (this.state.schema.widget === 'upload')
            type = UploadComponent;
        else if (this.state.schema.widget === 'upload64')
            type = UploadComponent;
        else if (this.state.schema.type === 'string')
            type = InputComponent;
        else if (this.state.schema.type === 'number')
            type = InputComponent;
        else if (this.state.schema.type === 'integer')
            type = InputComponent;
        else if (this.state.schema.type === 'boolean')
            type = BooleanComponent;
        else
            throw new Error(JSON.stringify(this.state.schema));
        const componentRef = viewContainerRef.createComponent(type);
        componentRef.instance.state = this.state;
    }
}
WrapperComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: WrapperComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
WrapperComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: WrapperComponent, selector: "app-wrapper", viewQueries: [{ propertyName: "compHost", first: true, predicate: CompDirective, descendants: true, static: true }], usesInheritance: true, ngImport: i0, template: "<span [matTooltip]=\"state.schema.description ? state.schema.description : ''\">\n    <ng-template compHost [matTooltip]=\"state.schema.description ? state.schema.description : ''\"></ng-template>\n</span>", styles: [""], dependencies: [{ kind: "directive", type: i1$2.MatTooltip, selector: "[matTooltip]", exportAs: ["matTooltip"] }, { kind: "directive", type: CompDirective, selector: "[compHost]" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: WrapperComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-wrapper', template: "<span [matTooltip]=\"state.schema.description ? state.schema.description : ''\">\n    <ng-template compHost [matTooltip]=\"state.schema.description ? state.schema.description : ''\"></ng-template>\n</span>" }]
        }], propDecorators: { compHost: [{
                type: ViewChild,
                args: [CompDirective, { static: true }]
            }] } });

class AdditionalPropertiesComponent extends BaseComponent {
    constructor() {
        super(...arguments);
        this.hover = null;
        /**
         * child states
         */
        this.states = [];
    }
    /**
     * populate FormGroup
     */
    ngOnInit() {
        super.ngOnInit();
        if (this.state.value)
            for (const [k, v] of Object.entries(this.state.value)) {
                const control = BaseComponent.createControl(this.state.schema.additionalProperties, v, this.state.schema.required ? this.state.schema.required.includes(k) : false);
                this.formGroup().addControl(k, control);
                this.states.push({
                    name: k,
                    schema: this.state.schema.additionalProperties,
                    value: v,
                    control
                });
            }
    }
    change(event, i) {
        const control = BaseComponent.createControl(this.state.schema.additionalProperties, undefined, false);
        const oldKey = Object.keys(this.formGroup().value)[i];
        const newKey = event.target.value;
        const state = {
            name: newKey,
            schema: this.state.schema.additionalProperties,
            value: undefined,
            control
        };
        this.states.splice(i, 1, state);
        this.formGroup().addControl(newKey, control);
        this.formGroup().removeControl(oldKey);
    }
    add() {
        if (Object.keys(this.formGroup().value).includes(''))
            // already contains empty key
            return;
        const control = BaseComponent.createControl(this.state.schema.additionalProperties, undefined, false);
        this.states.push({
            name: '',
            schema: this.state.schema.additionalProperties,
            value: undefined,
            control
        });
        this.formGroup().addControl('', control);
    }
    remove(i) {
        const key = Object.keys(this.formGroup().value)[i];
        this.formGroup().removeControl(key);
        this.states.splice(i, 1);
    }
}
AdditionalPropertiesComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: AdditionalPropertiesComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AdditionalPropertiesComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: AdditionalPropertiesComponent, selector: "app-additional-properties", usesInheritance: true, ngImport: i0, template: "<span *ngFor=\"let s of states; let i = index\">\n    <span (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\n        <mat-form-field>\n            <mat-label>{{state.name}}</mat-label>\n            <input matInput [value]=\"s.name\" (change)=\"change($event, i)\">\n        </mat-form-field>\n        &nbsp;\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n        &nbsp;\n    </span>\n</span>\n<button mat-icon-button (click)=\" add()\">\n    <mat-icon>add_circle_outline</mat-icon>\n</button>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i3.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i4.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i4.MatLabel, selector: "mat-label" }, { kind: "component", type: i2$2.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "component", type: i3$1.MatIconButton, selector: "button[mat-icon-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "component", type: WrapperComponent, selector: "app-wrapper" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: AdditionalPropertiesComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-additional-properties', template: "<span *ngFor=\"let s of states; let i = index\">\n    <span (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\n        <mat-form-field>\n            <mat-label>{{state.name}}</mat-label>\n            <input matInput [value]=\"s.name\" (change)=\"change($event, i)\">\n        </mat-form-field>\n        &nbsp;\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n        &nbsp;\n    </span>\n</span>\n<button mat-icon-button (click)=\" add()\">\n    <mat-icon>add_circle_outline</mat-icon>\n</button>" }]
        }] });

/**
 * renders arrays as a list of subforms with + and - buttons
 */
class ArrayComponent extends BaseComponent {
    constructor() {
        super(...arguments);
        /**
         * child states
         */
        this.states = [];
        /**
         * which subform is active
         */
        this.hover = null;
    }
    /**
     * populate FormArray control
     */
    ngOnInit() {
        super.ngOnInit();
        let i = 0;
        if (this.state.schema.items && this.state.value)
            for (const v of this.state.value) {
                const control = BaseComponent.createControl(this.state.schema.items, v, false);
                this.formArray().setControl(i, control);
                this.states.push({
                    name: this.state.name,
                    schema: this.state.schema.items,
                    value: v,
                    control
                });
                i++;
            }
    }
    /**
     * add a new element
     */
    add() {
        const control = BaseComponent.createControl(this.state.schema.items, undefined, false);
        this.formArray().setControl(this.states.length, control);
        this.states.push({
            name: this.state.name,
            schema: this.state.schema.items,
            value: undefined,
            control
        });
    }
    /**
     * remove element at index i
     */
    remove(i) {
        this.states.splice(i, 1);
        this.formArray().removeAt(i);
    }
}
ArrayComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: ArrayComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
ArrayComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: ArrayComponent, selector: "app-array", usesInheritance: true, ngImport: i0, template: "<ng-container *ngIf=\"state.schema.layout === 'vertical'\">\n    <div *ngFor=\"let s of states; let i = index\" (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n    </div>\n</ng-container>\n<ng-container *ngIf=\"state.schema.layout !== 'vertical'\">\n    <span *ngFor=\"let s of states; let i = index\" (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n    </span>\n</ng-container>\n<button mat-icon-button (click)=\" add()\">\n    <mat-icon>add_circle_outline</mat-icon>\n</button>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "component", type: i2$2.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "component", type: i3$1.MatIconButton, selector: "button[mat-icon-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "component", type: WrapperComponent, selector: "app-wrapper" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: ArrayComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-array', template: "<ng-container *ngIf=\"state.schema.layout === 'vertical'\">\n    <div *ngFor=\"let s of states; let i = index\" (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n    </div>\n</ng-container>\n<ng-container *ngIf=\"state.schema.layout !== 'vertical'\">\n    <span *ngFor=\"let s of states; let i = index\" (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n    </span>\n</ng-container>\n<button mat-icon-button (click)=\" add()\">\n    <mat-icon>add_circle_outline</mat-icon>\n</button>" }]
        }] });

/**
 * static JsonPointer implementation
 */
class JsonPointer {
    /**
     * evaluate the JSON pointer on o
     */
    static jsonPointer(o, pointer) {
        return JsonPointer.jsonPointer2(o, JsonPointer.split(pointer));
    }
    /**
     * evaluate the JSON pointer (parsed array of paths) on o
     */
    static jsonPointer2(o, paths) {
        if (o === undefined) {
            return undefined;
        }
        if (paths.length === 0) {
            return o;
        }
        const path = paths[0];
        const np = Object.assign([], paths);
        np.splice(0, 1);
        if (paths[0] === '*') {
            const res = [];
            for (const f of (typeof (o) === 'object' ? Object.values(o) : o)) {
                res.push(this.jsonPointer2(f, np));
            }
            return res;
        }
        else {
            return this.jsonPointer2(o[path], np);
        }
    }
    /**
     * strip leading / and split the JSON pointer
     */
    static split(s) {
        if (s === '') {
            return [];
        }
        if (s.startsWith('/')) {
            s = s.substring(1);
            const arr = s.split('/');
            for (const a of arr) {
                if (a === '') {
                    throw new Error('JSON Pointer must not contain an empty reference token');
                }
            }
            return arr;
        }
        throw new Error('JSON Pointer must start with /');
    }
}

class ObjectComponent extends BaseComponent {
    constructor() {
        super(...arguments);
        /**
         * child states
         */
        this.states = [];
    }
    /**
     * populate FormGroup
     */
    ngOnInit() {
        super.ngOnInit();
        if (this.state.schema.properties)
            for (const [k, v] of Object.entries(this.state.schema.properties)) {
                const control = BaseComponent.createControl(v, this.state.value?.[k], this.state.schema.required ? this.state.schema.required.includes(k) : false);
                this.formGroup().addControl(k, control);
                this.states.push({
                    name: k,
                    schema: v,
                    value: this.state.value?.[k],
                    control
                });
            }
    }
    show(i) {
        if (this.state.schema.switch) {
            const switc = this.formGroup().get(this.state.schema.switch)?.value;
            const prop = Object.values(this.state.schema.properties)[i];
            if (!prop.case)
                return true;
            if (!switc)
                return false;
            return (prop.case && prop.case.includes(switc));
        }
        return true;
    }
}
ObjectComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: ObjectComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
ObjectComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: ObjectComponent, selector: "app-object", usesInheritance: true, ngImport: i0, template: "<ng-container *ngIf=\"state.schema.layout === 'vertical'\">\n    <div *ngFor=\"let s of states; let i = index\">\n        <app-wrapper *ngIf=\"show(i)\" [state]=\"s\"></app-wrapper>\n        &nbsp;\n    </div>\n</ng-container>\n<ng-container *ngIf=\"state.schema.layout !== 'vertical'\">\n    <span *ngFor=\"let s of states; let i = index\">\n        <app-wrapper *ngIf=\"show(i)\" [state]=\"s\"></app-wrapper>\n        &nbsp;\n    </span>\n</ng-container>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "component", type: WrapperComponent, selector: "app-wrapper" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: ObjectComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-object', template: "<ng-container *ngIf=\"state.schema.layout === 'vertical'\">\n    <div *ngFor=\"let s of states; let i = index\">\n        <app-wrapper *ngIf=\"show(i)\" [state]=\"s\"></app-wrapper>\n        &nbsp;\n    </div>\n</ng-container>\n<ng-container *ngIf=\"state.schema.layout !== 'vertical'\">\n    <span *ngFor=\"let s of states; let i = index\">\n        <app-wrapper *ngIf=\"show(i)\" [state]=\"s\"></app-wrapper>\n        &nbsp;\n    </span>\n</ng-container>" }]
        }] });

class TabComponent extends ArrayComponent {
}
TabComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: TabComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TabComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: TabComponent, selector: "app-tab", usesInheritance: true, ngImport: i0, template: "<mat-tab-group>\n    <mat-tab *ngFor=\"let s of states; let i = index\" [label]=\"state.name + ' ' + (i+1)\">\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n    </mat-tab>\n    <mat-tab>\n        <ng-template mat-tab-label>\n            <button mat-icon-button (click)=\" add()\">\n                <mat-icon>add_circle_outline</mat-icon>\n            </button>\n        </ng-template>\n    </mat-tab>\n</mat-tab-group>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "component", type: i2$2.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "component", type: i3$1.MatIconButton, selector: "button[mat-icon-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "directive", type: i4$4.MatTabLabel, selector: "[mat-tab-label], [matTabLabel]" }, { kind: "component", type: i4$4.MatTab, selector: "mat-tab", inputs: ["disabled"], exportAs: ["matTab"] }, { kind: "component", type: i4$4.MatTabGroup, selector: "mat-tab-group", inputs: ["color", "disableRipple", "fitInkBarToContent", "mat-stretch-tabs"], exportAs: ["matTabGroup"] }, { kind: "component", type: WrapperComponent, selector: "app-wrapper" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: TabComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-tab', template: "<mat-tab-group>\n    <mat-tab *ngFor=\"let s of states; let i = index\" [label]=\"state.name + ' ' + (i+1)\">\n        <app-wrapper [state]=\"s\"></app-wrapper>\n        <button mat-icon-button (click)=\"remove(i)\">\n            <mat-icon>remove_circle_outline</mat-icon>\n        </button>\n    </mat-tab>\n    <mat-tab>\n        <ng-template mat-tab-label>\n            <button mat-icon-button (click)=\" add()\">\n                <mat-icon>add_circle_outline</mat-icon>\n            </button>\n        </ng-template>\n    </mat-tab>\n</mat-tab-group>" }]
        }] });

class TableComponent extends ArrayComponent {
    constructor() {
        super(...arguments);
        this.rows = [];
    }
    /**
     * populate FormArray control
     */
    ngOnInit() {
        super.ngOnInit();
        for (const state of this.states) {
            const object = new ObjectComponent(this.http, this.service);
            object.state = state;
            object.ngOnInit();
            this.rows.push(object);
        }
    }
    /**
     * add a new element
     */
    add() {
        super.add();
        const object = new ObjectComponent(this.http, this.service);
        object.state = this.states[this.states.length - 1];
        object.ngOnInit();
        this.rows.push(object);
    }
    remove(i) {
        super.remove(i);
        this.rows.splice(i, 1);
    }
}
TableComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: TableComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TableComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: TableComponent, selector: "app-table", usesInheritance: true, ngImport: i0, template: "<table>\n    <tr>\n        <th *ngFor=\"let x of state.schema.items?.properties | keyvalue:originalOrder\">{{x.value.title ? x.value.title :\n            x.key}}\n        </th>\n        <th id=\"remove_row\"></th>\n    </tr>\n    <tr *ngFor=\"let s of states; let i = index\">\n        <td *ngFor=\"let x of state.schema.items?.properties | keyvalue:originalOrder; let column = index\">\n            <app-wrapper [state]=\"rows[i].states[column]\"></app-wrapper>\n        </td>\n        <td>\n            <button mat-icon-button (click)=\"remove(i)\">\n                <mat-icon>remove_circle_outline</mat-icon>\n            </button>\n        </td>\n    </tr>\n</table>\n<button mat-icon-button (click)=\" add()\">\n    <mat-icon>add_circle_outline</mat-icon>\n</button>", styles: [""], dependencies: [{ kind: "directive", type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "component", type: i2$2.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "component", type: i3$1.MatIconButton, selector: "button[mat-icon-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "component", type: WrapperComponent, selector: "app-wrapper" }, { kind: "pipe", type: i1$1.KeyValuePipe, name: "keyvalue" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: TableComponent, decorators: [{
            type: Component,
            args: [{ selector: 'app-table', template: "<table>\n    <tr>\n        <th *ngFor=\"let x of state.schema.items?.properties | keyvalue:originalOrder\">{{x.value.title ? x.value.title :\n            x.key}}\n        </th>\n        <th id=\"remove_row\"></th>\n    </tr>\n    <tr *ngFor=\"let s of states; let i = index\">\n        <td *ngFor=\"let x of state.schema.items?.properties | keyvalue:originalOrder; let column = index\">\n            <app-wrapper [state]=\"rows[i].states[column]\"></app-wrapper>\n        </td>\n        <td>\n            <button mat-icon-button (click)=\"remove(i)\">\n                <mat-icon>remove_circle_outline</mat-icon>\n            </button>\n        </td>\n    </tr>\n</table>\n<button mat-icon-button (click)=\" add()\">\n    <mat-icon>add_circle_outline</mat-icon>\n</button>" }]
        }] });

/**
 * entry component
 */
class JsonSchemaFormComponent {
    /**
     * register container form elements to avoid cyclic imports
     */
    ngOnInit() {
        const clone = JSON.parse(JSON.stringify(this.state.schema));
        this.resolvedState = {
            value: this.state.value,
            schema: clone,
            control: this.state.control,
            name: this.state.name
        };
        WrapperComponent.arrayComponent = ArrayComponent;
        WrapperComponent.tabComponent = TabComponent;
        WrapperComponent.tableComponent = TableComponent;
        WrapperComponent.objectComponent = ObjectComponent;
        WrapperComponent.additionalPropertiesComponent = AdditionalPropertiesComponent;
        this.resolve(clone);
        BaseComponent.prepareControl(this.state.control, clone, this.state.value, false);
    }
    resolve(schema) {
        if (!schema)
            return;
        if (schema.$ref) {
            if (schema.$ref.startsWith('#'))
                for (const [k, v] of Object.entries(JsonPointer.jsonPointer(this.state.schema, schema.$ref.substring(1))))
                    schema[k] = v;
        }
        this.resolve(schema.additionalProperties);
        this.resolve(schema.items);
        if (schema.properties)
            for (const prop of Object.values(schema.properties))
                this.resolve(prop);
    }
}
JsonSchemaFormComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
JsonSchemaFormComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.2.3", type: JsonSchemaFormComponent, selector: "lib-json-schema-form", inputs: { state: "state" }, ngImport: i0, template: `
    <app-wrapper [state]="resolvedState"></app-wrapper>
  `, isInline: true, dependencies: [{ kind: "component", type: WrapperComponent, selector: "app-wrapper" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-json-schema-form', template: `
    <app-wrapper [state]="resolvedState"></app-wrapper>
  ` }]
        }], propDecorators: { state: [{
                type: Input
            }] } });

class JsonSchemaFormModule {
}
JsonSchemaFormModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
JsonSchemaFormModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormModule, declarations: [JsonSchemaFormComponent,
        BooleanComponent,
        InputComponent,
        ObjectComponent,
        BaseComponent,
        SelectComponent,
        ArrayComponent,
        UploadComponent,
        DateComponent,
        TextareaComponent,
        AdditionalPropertiesComponent,
        AutocompleteComponent,
        TabComponent,
        TableComponent,
        ChipsComponent,
        CompDirective,
        WrapperComponent], imports: [CommonModule,
        HttpClientModule,
        FormsModule,
        ReactiveFormsModule,
        MatCheckboxModule,
        MatInputModule,
        MatIconModule,
        MatButtonModule,
        MatSelectModule,
        MatDatepickerModule,
        MatNativeDateModule,
        MatTooltipModule,
        MatAutocompleteModule,
        MatTabsModule,
        MatChipsModule], exports: [JsonSchemaFormComponent,
        BaseComponent] });
JsonSchemaFormModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormModule, imports: [CommonModule,
        HttpClientModule,
        FormsModule,
        ReactiveFormsModule,
        MatCheckboxModule,
        MatInputModule,
        MatIconModule,
        MatButtonModule,
        MatSelectModule,
        MatDatepickerModule,
        MatNativeDateModule,
        MatTooltipModule,
        MatAutocompleteModule,
        MatTabsModule,
        MatChipsModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.3", ngImport: i0, type: JsonSchemaFormModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        JsonSchemaFormComponent,
                        BooleanComponent,
                        InputComponent,
                        ObjectComponent,
                        BaseComponent,
                        SelectComponent,
                        ArrayComponent,
                        UploadComponent,
                        DateComponent,
                        TextareaComponent,
                        AdditionalPropertiesComponent,
                        AutocompleteComponent,
                        TabComponent,
                        TableComponent,
                        ChipsComponent,
                        CompDirective,
                        WrapperComponent
                    ],
                    imports: [
                        CommonModule,
                        HttpClientModule,
                        FormsModule,
                        ReactiveFormsModule,
                        MatCheckboxModule,
                        MatInputModule,
                        MatIconModule,
                        MatButtonModule,
                        MatSelectModule,
                        MatDatepickerModule,
                        MatNativeDateModule,
                        MatTooltipModule,
                        MatAutocompleteModule,
                        MatTabsModule,
                        MatChipsModule
                    ],
                    exports: [
                        JsonSchemaFormComponent,
                        BaseComponent
                    ]
                }]
        }] });

class State {
}

/*
 * Public API Surface of json-schema-form
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BaseComponent, JsonSchemaFormComponent, JsonSchemaFormModule, JsonSchemaFormService, State };
//# sourceMappingURL=tbotlabs-json-schema-form.mjs.map
